#!/bin/bash

basedir=`dirname "$0"`
version="1.0.26"
this=`basename "$0"`

yellow_color_switch_on () {
    type tput &>/dev/null && tput setaf 3 || echo "";
}

green_color_switch_on () {
    type tput &>/dev/null && tput setaf 2 || echo "";
}

red_color_switch_on () {
    type tput &>/dev/null && tput setaf 1 || echo "";
}

color_switch_off() {
    type tput &>/dev/null && tput sgr0 || echo "";
}

usage () {
   cat <<EOF

Usage:
  $this [options]

  Current version of C++TESK Testing ToolKit

  Options:
    --new                     - find what is new in C++TESK $version
    --log                     - show the complete C++TESK change log
    --version | -v            - show current version of the toolkit and its components (default)
    --help | -h               - show this message

EOF
}

version () {
  if [ -f $basedir/../Version ]
  then
    echo
    yellow_color_switch_on
    cat $basedir/../Version
    color_switch_off
    echo
    echo Use --new option to find what is new in C++TESK Testing ToolKit $version
    echo
  else
    red_color_switch_on
    echo "[ERROR] File \"Version\" is missing!"
    color_switch_off
  fi
}

new_info () {
  if [ -f $basedir/../ChangeLog ]
  then
    echo
    yellow_color_switch_on
    head $basedir/../ChangeLog
    color_switch_off
    echo
    echo Use --log option to get the complete C++TESK Testing ToolKit change log
    echo
  else
    red_color_switch_on
    echo "[ERROR] File \"ChangeLog\" is missing!"
    color_switch_off
  fi
}

all_info () {
  if [ -f $basedir/../ChangeLog ]
  then
    echo
    yellow_color_switch_on
    cat $basedir/../ChangeLog
    color_switch_off
    echo
  else
    red_color_switch_on
    echo "[ERROR] File \"ChangeLog\" is missing!"
    color_switch_off
  fi
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --version | -v )
      version
      exit 0
    ;;
    --help | -h )
      usage
      exit 0
    ;;
    --new )
      new_info
      exit 0
    ;;
    --log )
      all_info
      exit 0
    ;;
    --help | -h | * )
      usage
      exit 0
    ;;
  esac
  shift
done

version

