#include "account_model.hpp"
#include <iostream>

AccountModel::BalanceState cov_BalanceState(int balance, int limit)
{
    return (balance > 0)? AccountModel::BalanceState::POSITIVE :
           (balance == 0)? AccountModel::BalanceState::ZERO :
           (balance > limit)? AccountModel::BalanceState::NEGATIVE :
           (balance == limit)? AccountModel::BalanceState::LIMIT_REACHED
           : AccountModel::BalanceState::OVERDRAFT;
}

AccountModel::DepositStart AccountModel::cov_DepositStart() const
{
    return cov_BalanceState(this->balance, this->limit);
}
AccountModel::DepositEnd AccountModel::cov_DepositEnd(int sum) const
{
    return cov_BalanceState(this->balance+sum, this->limit);
}
AccountModel::WithdrawStart AccountModel::cov_WithdrawStart() const
{
    return cov_BalanceState(this->balance, this->limit);
}
AccountModel::WithdrawEnd AccountModel::cov_WithdrawEnd(int sum) const
{
    return cov_BalanceState(this->balance-sum, this->limit);
}

AccountModel::Deposit AccountModel::cov_Deposit(int sum) const
{
    return cov_DepositStart() * cov_DepositEnd(sum);
}
AccountModel::Withdraw AccountModel::cov_Withdraw(int sum) const
{
    return cov_WithdrawStart() * cov_WithdrawEnd(sum);
}

bool AccountModel::pre_deposit(int sum) const
{
    return sum >= 0;
}

void AccountModel::cov_deposit(int sum) {
//	coverage[cov_DepositStart()]++;
//	coverage[cov_DepositEnd(sum)]++;
//	coverage[cov_Deposit(sum)]++;
	coverage << cov_DepositStart();
	coverage << cov_DepositEnd(sum);
	coverage << cov_Deposit(sum);
}

bool AccountModel::post_deposit(int pre_balance, int sum, int result)
{
    return (balance == pre_balance+result) && (result >= 0) && (result <= sum);
}

bool AccountModel::pre_withdraw(int sum) const
{
    return sum >= 0;
}

void AccountModel::cov_withdraw(int sum) {
	WithdrawStart ws;
	ws = cov_WithdrawStart();
	coverage[ws]++;
//	coverage << ws;
	WithdrawEnd we;
	we = cov_WithdrawEnd(sum);
	coverage[we]++;
	coverage[cov_Withdraw(sum)]++;
//	coverage << we;
//	coverage << cov_Withdraw(sum);
}

bool AccountModel::post_withdraw(int pre_balance, int sum, int result)
{
    return (balance == pre_balance-result) && (result >= 0) && (result <= sum) && (balance >= limit);
}

AccountModel::AccountModel(void) : balance(0), limit(0), coverage(true) {}
AccountModel::AccountModel(int limit) : balance(0), limit(limit), coverage(true) {}
AccountModel::~AccountModel(void) {}

bool AccountModel::post_get_balance(int pre_balance, int result)
{
    return pre_balance == balance && result == balance;
}
