/*
 * Copyright 2012-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK (cpptesk::examples)
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Test configuration.
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/field.hpp>
#include <hw/utils/singleton.hpp>
#include <ts/config.hpp>

using namespace cpptesk::hw;
using namespace cpptesk::ts;

namespace cpptesk {
namespace examples {
namespace moesi {

/**
 * Testbench configuration.
 */
class MoesiConfig: public Config, public Singleton<MoesiConfig>
{
    friend class Singleton<MoesiConfig>;

    /// Configuration constructor.
    MoesiConfig()
    {
        CPPTESK_ADD_OPTION(cores_number);
        CPPTESK_ADD_OPTION(cache_protocol);
        CPPTESK_ADD_OPTION(state_abstraction);
        CPPTESK_ADD_OPTION(stimulus_abstraction);
	CPPTESK_ADD_OPTION(read_enable);
	CPPTESK_ADD_OPTION(write_enable);
	CPPTESK_ADD_OPTION(evict_enable);
	CPPTESK_ADD_OPTION(reset_enable);
    }

public:

    ///////////////////////////////////////////////////////////////////////////
    // Number of cores
    ///////////////////////////////////////////////////////////////////////////
    const static size_t default_cores_number = 4;
    size_t cores_number;

    ///////////////////////////////////////////////////////////////////////////
    // Cache coherence protocol:
    //
    // 0) MSI   (Modified,                   Shared, Invalid)
    // 1) MOSI  (Modified, Owned,            Shared, Invalid)
    // 2) MESI  (Modified,        Exclusive, Shared, Invalid)
    // 3) MOESI (Modified, Owned, Exclusive, Shared, Invalid)
    ///////////////////////////////////////////////////////////////////////////
    const static size_t default_cache_protocol = 0;
    size_t cache_protocol;

    ///////////////////////////////////////////////////////////////////////////
    // State abstraction technique:
    //
    // 0) VECTOR   - an abstract state is a vector of cache line states.
    // 1) MULTISET - an abstract state is a multiset of cache line states.
    // 2) SET      - an abstract state is a set of cache line states.
    ///////////////////////////////////////////////////////////////////////////
    const static size_t default_state_abstraction = 0;
    size_t state_abstraction;

    ///////////////////////////////////////////////////////////////////////////
    // Stimulus abstraction technique:
    //
    // 0) INDEX - an abstract stimulus is a pair < operation, core index >.
    // 1) STATE - an abstract stimulus is a pair < operation, core state >.
    ///////////////////////////////////////////////////////////////////////////
    const static size_t default_stimulus_abstraction = 0;
    size_t stimulus_abstraction;

    ///////////////////////////////////////////////////////////////////////////
    // Read operation 
    ///////////////////////////////////////////////////////////////////////////
    const static bool default_read_enable = true;
    bool read_enable;

    ///////////////////////////////////////////////////////////////////////////
    // Write operation 
    ///////////////////////////////////////////////////////////////////////////
    const static bool default_write_enable = true;
    bool write_enable;

    ///////////////////////////////////////////////////////////////////////////
    // Eviction operation 
    ///////////////////////////////////////////////////////////////////////////
    const static bool default_evict_enable = true;
    bool evict_enable;

    ///////////////////////////////////////////////////////////////////////////
    // Reset operation 
    ///////////////////////////////////////////////////////////////////////////
    const static bool default_reset_enable = false;
    bool reset_enable;
};

}}} // cpptesk::examples::moesi

