/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/diagnostics/rules/rule.hpp>
#include <hw/diagnostics/metrics/metric.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

enum MaxReactionNumbers {
    MAX_FUZZY_RULE_NUMBER = 5,
    MAX_NORMAL_RULE_NUMBER = 6
};

struct FuzzyMatchResult {
    int weight;
    int rule_number;
};

class ErrorModel //: public Rule
{
public:
    /// Constructor of ErrorModel.
    ErrorModel();

    /// Destructor of ErrorModel.
    virtual ~ErrorModel();

    /// Transforms two reaction pairs by means of the given rule.
    ReactionUnitVector apply(ReactionUnit &ru1, ReactionUnit &ru2, uint rule_number, CollapseType type);

    /// Returns distance between two reaction pairs according to the given rule.
    int check(cpptesk::hw::Reaction &r1, cpptesk::hw::Reaction &r2, uint rule_number, CollapseType type);

    int check(ReactionUnit &ru1, ReactionUnit &ru2, uint rule_number, CollapseType type);

    /// Returns synopsis of rule with given number and type.
    //std::string toString(uint rule_number, bool fuzzy_logic);
protected:
    /// Rules for reactions equal in data
    std::vector<Rule*> _rules;
};

}}}
