/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Model of an operation execution process
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

namespace cpptesk {
namespace hw {

#define __CPPTESK_DEFINE_PRIORITY(name, value) \
    PRIORITY_##name = LOWEST_PRIORITY + value

#define __CPPTESK_DEFINE_10_PRIORITIES(prefix, value) \
    __CPPTESK_DEFINE_PRIORITY(prefix##0, value + 0), \
    __CPPTESK_DEFINE_PRIORITY(prefix##1, value + 1), \
    __CPPTESK_DEFINE_PRIORITY(prefix##2, value + 2), \
    __CPPTESK_DEFINE_PRIORITY(prefix##3, value + 3), \
    __CPPTESK_DEFINE_PRIORITY(prefix##4, value + 4), \
    __CPPTESK_DEFINE_PRIORITY(prefix##5, value + 5), \
    __CPPTESK_DEFINE_PRIORITY(prefix##6, value + 6), \
    __CPPTESK_DEFINE_PRIORITY(prefix##7, value + 7), \
    __CPPTESK_DEFINE_PRIORITY(prefix##8, value + 8), \
    __CPPTESK_DEFINE_PRIORITY(prefix##9, value + 9)

#define __CPPTESK_DEFINE_100_PRIORITIES(prefix, value) \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##0, value + 00), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##1, value + 10), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##2, value + 20), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##3, value + 30), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##4, value + 40), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##5, value + 50), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##6, value + 60), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##7, value + 70), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##8, value + 80), \
    __CPPTESK_DEFINE_10_PRIORITIES(prefix##9, value + 90)

#define __CPPTESK_PRIORITY_MASK 0xff

/** @typedef priority_t
 * Enumerated constants for process priorities.
 */
typedef enum priority_t
{
    //-------------------------------------------------------------------------
    // Default Priority
    //-------------------------------------------------------------------------

    /// The reserved value that is equivalent to NORMAL_PRIORITY.
    NULL_PRIORITY    = 0,

    //-------------------------------------------------------------------------
    // Basic Priorities (LOWEST < LOW < NORMAL < HIGH < HIGHEST)
    //-------------------------------------------------------------------------

    /// The lowest priority.
    LOWEST_PRIORITY  = 10,
     /// The highest priority.
    HIGHEST_PRIORITY = LOWEST_PRIORITY + 99,
    /// The low priority.
    LOW_PRIORITY     = LOWEST_PRIORITY + 1,
    /// The normal (medium) value of priority.
    NORMAL_PRIORITY  = (LOWEST_PRIORITY + HIGHEST_PRIORITY) / 2,
    /// The high priority.
    HIGH_PRIORITY    = HIGHEST_PRIORITY - 1,

    //-------------------------------------------------------------------------
    // All Priorities (LOWEST=PRIORITY_00, ..., PRIORITY_99=HIGHEST)
    //-------------------------------------------------------------------------

    /// PRIORITY_00, ..., PRIORITY_99
    __CPPTESK_DEFINE_100_PRIORITIES(,  000),
    /// PRIORITY_000, ..., PRIORITY_099 = PRIORITY_00, ..., PRIORITY_99
    __CPPTESK_DEFINE_100_PRIORITIES(0, 000),

    //-------------------------------------------------------------------------
    // System Priorities (For Internal Tool Usage)
    //-------------------------------------------------------------------------

    /// The truly lowest priority.
    _REAL_LOWEST_PRIORITY  = 0x01,
    /// The truly highest priority.
    _REAL_HIGHEST_PRIORITY = __CPPTESK_PRIORITY_MASK

} priority_t;

inline unsigned get_real_priority(unsigned priority)
{
    return priority ? priority : (unsigned)NORMAL_PRIORITY;
}

}} // cpptesk::hw

