#!/bin/bash

PORT_NUMBER="2010"
CONF_DIR="conf"
CLUSTER_NAME="localhost"
TESTRUNS_DIR="testruns"
basedir=`dirname "$0"`
this=`basename "$0"`

usage () {
   cat <<EOF

Usage:
  $this [options]

  This script runs distributed on cluster traversing.

  Options:
    --port=port_number        - set port number (default value is 2010)
    --confdir=dirname         - set configuration folder name (default value is conf)
    --cluster=clustername     - set cluster name (default value is localhost)
    --testrunsdir=dirname     - set name of folder to contain information about test runnings (default value is testruns)
    --help | -h               - show this message

EOF
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --port | --port= )
      echo "--port option should be followed by a port number"
      echo ""
      usage
      exit 1
    ;;
    --confdir | --confdir= )
      echo "--confdir option should be followed by a folder name"
      echo ""
      usage
      exit 1
    ;;
    --cluster | --cluster= )
      echo "--cluster option should be followed by a cluster name"
      echo ""
      usage
      exit 1
    ;;
    --testrunsdir | --testrunsdir= )
      echo "--testrunsdir option should be followed by a folder name"
      echo ""
      usage
      exit 1
    ;;

    --port=* )
      value=`echo $1 | sed "s/--port=//"`
      PORT_NUMBER="$value"
    ;;
    --confdir=* )
      value=`echo $1 | sed "s/--confdir=//"`
      CONF_DIR="$value"
    ;;
    --cluster=* )
      value=`echo $1 | sed "s/--cluster=//"`
      CLUSTER_NAME="$value"
    ;;
    --testrunsdir=* )
      value=`echo $1 | sed "s/--testrunsdir=//"`
      TESTRUNS_DIR="$value"
    ;;
    --help | -h )
      usage
      exit 0
    ;;
    * )
      usage
      exit 0
  esac
  shift
done

export "LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$basedir/src/mhttpd/libmicrohttpd/lib" \
   && cd $basedir && ./httpmcc $PORT_NUMBER $CONF_DIR $CLUSTER_NAME $TESTRUNS_DIR
